
{-----------------------------------------------------------------------------|
| COLUMNS.PAS  -  divides an one column text in several columns.  |
|                 Parameters:  p1  Number of columns  |
|                              p2  Pagelength  |
|  |
| To be used with DCL Procedure PIPE.COM.                (Author: T.HALL)  |
|  |
|-----------------------------------------------------------------------------}

PROGRAM columns (input, output);

CONST string_length = 984; { Length chosen because of TPU }
      blank = ' ';
      tab = chr (9);
      page_length = 62;
      line_delimiter = chr (13) + blank + chr (10);
      delim_length = length (line_delimiter);

TYPE  line_type = VARYING [string_length] OF char;
      column    = RECORD
                     max_length : integer;
                     next : ^column
                  END;

%INCLUDE 'parm.def'      { Allows to get next parameter with proc. NEXT_PARM }

VAR   line, parm, out_line, blank_line : line_type;
      internal_file : FILE OF line_type;
      j, lines_per_page, text_columns : integer;
      first_column : ^column := NIL;


   PROCEDURE scan_line;

   VAR   last_column, next_column : ^column;
         start, i, col_lght,
         length : integer;
         ready : boolean;

   BEGIN {procedure scan_line}
      IF line <> '' THEN
      BEGIN                                                      
         length := line.length;
         i := 1;
         next_column := first_column;
         last_column := first_column;
         WHILE i < length DO
         BEGIN
            start := i;
            ready := false;
            REPEAT                        { scan characters <> line_delimiter }
               IF i <= length - delim_length THEN
                  IF substr (line, i, delim_length) <> line_delimiter THEN
                     i := succ (i)
                  ELSE
                     ready := true
               ELSE
                  ready := true;
            UNTIL ready;
            IF next_column = NIL THEN
            BEGIN
               new (next_column);
               WITH next_column@ DO
               BEGIN
                  next := NIL;
                  max_length := 0;
               END {with};
               IF first_column = NIL THEN
                  first_column := next_column
               ELSE
                  last_column@.next := next_column;
            END;
            last_column := next_column;
            IF i > length - delim_length THEN 
               col_lght := length - start + 1
            ELSE 
               col_lght := i - start;
            IF col_lght > last_column@.max_length THEN
               last_column@.max_length := col_lght;
            i := i + delim_length;
            next_column := next_column@.next;
         END {while};
      END {if};
   END {procedure scan_line};


   PROCEDURE adjust_line;

   CONST separators = blank + tab;

   VAR   next_column : ^column;
         start, i, col_lght,
         blanks, length : integer;
         ready : boolean;

   BEGIN {procedure adjust_line}
      out_line := '';
      IF line <> '' THEN
      BEGIN
         length := line.length;
         i := 1;
         next_column := first_column;
         WHILE i <= length DO
         BEGIN
            start := i;
            ready := false;
            REPEAT                        { scan characters <> line_delimiter }
               IF i <= length - delim_length THEN
                  IF substr (line, i, delim_length) <> line_delimiter THEN
                     i := succ (i)
                  ELSE
                     ready := true
               ELSE
                  ready := true;
            UNTIL ready;
            IF i > length - delim_length THEN 
               col_lght := length - start + 1
            ELSE 
               col_lght := i - start;
            IF col_lght > 0 THEN
               out_line := out_line + substr (line, start, col_lght);
            blanks := next_column@.max_length - col_lght;
            IF blanks > 0 THEN
               out_line := out_line + substr (blank_line, 1, blanks);
            out_line := out_line + blank;
            i := i + delim_length;
            next_column := next_column@.next;
         END {while};
         out_line.length := out_line.length - 1;
      END {if};
   END {procedure adjust_line};


   PROCEDURE build_pages;

   VAR   files : ARRAY [0..1] OF FILE OF line_type;
         first_file, second_file, lp, tc : integer;
         buffer : line_type;

   BEGIN {procedure build_pages}
      blank_line.length := string_length;
      FOR j := 1 TO string_length DO blank_line [j] := blank;
      rewrite (internal_file);
      rewrite (files [0]);
      WHILE NOT eof DO
      BEGIN
         FOR tc := 1 TO text_columns DO
            IF NOT eof THEN
            BEGIN
               first_file := (tc - 1) MOD 2;
               second_file := tc MOD 2;
               reset (files [first_file]);
               rewrite (files [second_file]);
               FOR lp := 1 TO lines_per_page DO
               BEGIN
                  IF NOT eof THEN
                     readln (line)
                  ELSE
                     line := '';
                  IF NOT eof (files [first_file]) THEN
                  BEGIN
                     IF line <> '' THEN
                        buffer := files [first_file]@ + line_delimiter
                     ELSE
                        buffer := files [first_file]@;
                     get (files [first_file]);
                  END
                  ELSE
                     buffer := '';
                  buffer := buffer + line;
                  files [second_file]@ := buffer;
                  put (files [second_file])
               END {for};
            END {if};
         reset (files [second_file]);
         WHILE NOT eof (files [second_file]) DO
         BEGIN
            line := files [second_file]@;
            scan_line;
            internal_file@ := line;
            put (internal_file);
            get (files [second_file])
         END {while};
         rewrite (files [0]);
         rewrite (files [1]);
      END {while};
   END {procedure build_pages};


BEGIN {program columns}
   next_parm (parm);
   IF parm = '' THEN
      writeln ('*** COLUMNS: Missing Parameter: How many columns')
   ELSE
   BEGIN
      readv (parm, text_columns);
      next_parm (parm);
      IF parm = '' THEN
         lines_per_page := page_length
      ELSE
         readv (parm, lines_per_page);
      build_pages;
      reset (internal_file);
      WHILE NOT eof (internal_file) DO
      BEGIN
         line := internal_file@;
         adjust_line;
         writeln (out_line);
         get (internal_file);
      END {while};
   END {if};
END. {program columns}
