
{-----------------------------------------------------------------------------|
| JUSTIFY.PAS  -  justifies a text written in several columns.  |
|  |
| To be used with DCL Procedure PIPE.COM.                (Author: T.HALL)  |
|  |
|-----------------------------------------------------------------------------}

PROGRAM justify (input, output);

CONST string_length = 984; { Length chosen because of TPU }
      blank = ' ';
      tab = chr (9);

TYPE  line_type = VARYING [string_length] OF char;
      column    = RECORD
                     max_length : integer;
                     next : ^column
                  END;

VAR   line, out_line, blank_line : line_type;
      internal_file: FILE OF line_type;
      j : integer;
      first_column : ^column := NIL;


   PROCEDURE scan_line;

   CONST separators = blank + tab;

   VAR   last_column, next_column : ^column;
         start, i, col_lght,
         length : integer;
         ready : boolean;

   BEGIN {procedure scan_line}
      IF line <> '' THEN
      BEGIN
         length := line.length;
         i := 1;
         next_column := first_column;
         last_column := first_column;
         WHILE i <= length DO
         BEGIN
            ready := false;
            REPEAT                        { scan separators  }
               IF i <= length THEN
                 IF index (separators, line [i]) > 0 THEN
                    i := succ (i)
                 ELSE
                    ready := true
               ELSE
                  ready := true;
            UNTIL ready;
            start := i;
            ready := false;
            REPEAT                        { scan characters <> separator  }
               IF i <= length THEN
                 IF index (separators, line [i]) = 0 THEN
                    i := succ (i)
                 ELSE
                    ready := true
               ELSE
                  ready := true;
            UNTIL ready;
            IF next_column = NIL THEN
            BEGIN
               new (next_column);
               WITH next_column@ DO
               BEGIN
                  next := NIL;
                  max_length := 0;
               END {with};
               IF first_column = NIL THEN
                  first_column := next_column
               ELSE
                  last_column@.next := next_column;
            END;
            last_column := next_column;
            col_lght := i - start;
            IF col_lght > last_column@.max_length THEN
               last_column@.max_length := col_lght;
            next_column := next_column@.next;
         END {while};
      END {if};
   END {procedure scan_line};

   PROCEDURE adjust_line;

   CONST separators = blank + tab;

   VAR   next_column : ^column;
         start, i, col_lght,
         blanks, length : integer;
         ready : boolean;

   BEGIN {procedure adjust_line}
      out_line := '';
      IF line <> '' THEN
      BEGIN
         length := line.length;
         i := 1;
         next_column := first_column;
         WHILE i <= length DO
         BEGIN
            ready := false;
            REPEAT                        { scan separators  }
               IF i <= length THEN
                 IF index (separators, line [i]) > 0 THEN
                    i := succ (i)
                 ELSE
                    ready := true
               ELSE
                  ready := true;
            UNTIL ready;
            start := i;
            ready := false;
            REPEAT                        { scan characters <> separator  }
               IF i <= length THEN
                 IF index (separators, line [i]) = 0 THEN
                    i := succ (i)
                 ELSE
                    ready := true
               ELSE
                  ready := true;
            UNTIL ready;
            col_lght := i - start;
            IF col_lght > 0 THEN
               out_line := out_line + substr (line, start, col_lght);
            blanks := next_column@.max_length - col_lght;
            IF blanks > 0 THEN
               out_line := out_line + substr (blank_line, 1, blanks);
            out_line := out_line + blank;
            next_column := next_column@.next;
         END {while};
         out_line.length := out_line.length - 1;
      END {if};
   END {procedure adjust_line};

BEGIN {program justify}
   rewrite (internal_file);
   WHILE NOT eof DO
   BEGIN
      readln (line);
      scan_line;
      internal_file@ := line;
      put (internal_file);
   END {while};
   reset (internal_file);
   blank_line.length := string_length;
   FOR j := 1 TO string_length DO blank_line [j] := blank;
   WHILE NOT eof (internal_file) DO
   BEGIN
      line := internal_file@;
      adjust_line;
      writeln (out_line);
      get (internal_file);
   END {while};
END. {program justify}
